## -*-CPerl-*-

package Voiceglue::Conf;

use Satc;

use Exporter 'import';

use warnings;
use strict;

=head1 NAME

Voiceglue::Conf - Support for reading/writing voiceglue.conf file

=head1 VERSION

Version 0.01

=cut

our $VERSION = '0.01';

=head1 SYNOPSIS

    use Voiceglue::Conf;

=head1 EXPORT

=head1 FUNCTIONS

=head2 ($ok, $msg, $conf_map) = parse_voiceglue_conf ($conf_file_path);

Given a voiceglue.conf file path (defaults to "/etc/voiceglue.conf",
returns a structure that contains:
 ->{"urls"}{<exten>}	-- URL of VXML to load when <exten> is called
 ->{"params"}{<param>}  -- value of parameter <param>
 ->{"lines"}[<line#>]	-- raw lines of text from the file
Returns $ok true on success, false on failure and sets $msg to an error msg.
Note that $conf_map may still be populated even if errors are found.

=cut

sub parse_voiceglue_conf
{
    my ($path) = shift (@_);
    my ($ok, $msg, $line, $orig_line, $name, @lines, $fields);
    my ($conf_map) = {"urls" => {}, "params" => {}, "lines" => []};
    my (@delayed_errors) = ();

    defined ($path) || ($path = "/etc/voiceglue.conf");
    open (::Voiceglue::Conf::Fh, $path)
      || return (0, "Cannot open \"$path\" for reading : $!");
    @lines = <::Voiceglue::Conf::Fh>;
    close (::Voiceglue::Conf::Fh);
    $conf_map->{"lines"} = [@lines];

    while (defined ($line = shift (@lines)))
    {
	##  Strip out all sorts of bad stuff like comments & blank lines
	chomp ($line);
	$orig_line = $line;
	$line =~ s/^\s+//s;
	$line =~ s/\s+$//s;
	$line =~ s/^\#.*//s;
	if (length ($line))
	{
	    if ($line =~ /^(\S+),?\s+(\S+)$/)
	    {
		##  A URL line
		$conf_map->{"urls"}{$1} = $2;
	    }
	    elsif ($line =~ /^\S+\s+=\s+\S/)
	    {
		##  A parameter (potentially)
		($ok, $msg, $fields) = Satc::_parse_SATC_fields ($line);
		if (! $ok)
		{
		    push (@delayed_errors, 
			  "Cannot parse line in \"$path\": $orig_line");
		}
		elsif (scalar (@$fields) == 3)
		{
		    $conf_map->{"params"}{$fields->[0]} = $fields->[2];
		}
		else
		{
		    $name = $fields->[0];
		    shift (@$fields);
		    shift (@$fields);
		    $conf_map->{"params"}{$name} = $fields;
		};
	    }
	    else
	    {
		push (@delayed_errors,
		      "Cannot parse line in \"$path\": $orig_line");
	    };
	};
    };

    return ((scalar (@delayed_errors)) ? 0 : 1,
	    join (", ", @delayed_errors),
	    $conf_map);
};


=head1 AUTHOR

Doug Campbell, C<< <voiceglue at voiceglue.org> >>

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Voiceglue::Conf

You can also look for information at:

=over 4

=item * VoiceGlue Home Page

L<http://voiceglue.org>

=back

=head1 COPYRIGHT & LICENSE

Copyright 2009 Ampersand Inc., and Doug Campbell

This file is part of Voiceglue::Conf.

Voiceglue::Conf is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Voiceglue::Conf is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Voiceglue::Conf; if not, see <http://www.gnu.org/licenses/>.

=cut

1; # End of Voiceglue::Conf
